!---------------------------------------------------------------------------------------
! PREP_REGIONS: mod_var
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> mod_var 
!!
!---------------------------------------------------------------------------------------

module mod_var

  implicit none

  ! fixed parameters 
  ! -----------------

  integer, parameter :: max_path_len=200         ! max character length of paths and files
  integer, parameter :: max_name_len=50          ! max character length of variable names
  integer, parameter :: recname_len=7            ! length of receptor names
  real, parameter    :: numscale=1.e12           ! numeric scaling factor
  integer, parameter :: maxpoint=1000            ! max number of releases per receptor per month
  integer, parameter :: maxspec=10               ! max number of species in a flexpart run
  integer, parameter :: maxlev=50                ! max number of vertical levels in flexpart output
  integer, parameter :: maxobs=10000             ! max number of observations

  ! flexpart variables 
  ! ------------------

  integer            :: ageclass                 ! length of backtrajectories (s)
  integer            :: trajdays                 ! length of backtrajectories (days)
  integer            :: maxngrid                 ! max number of footprints (ndgrid*trajdays+2)
  integer            :: ndgrid                   ! number of footprints per day
  integer            :: nxgrid                   ! number of longitudinal grid cells
  integer            :: nygrid                   ! number of latitudinal grid cells
  integer            :: nzgrid                   ! number of vertical layers in grid_init files
  integer            :: nzlev                    ! number of vertical layers in grid_time files (nzlev = 1)
  real               :: llx                      ! lower right longitude of flexpart grid
  real               :: lly                      ! lower right latitude of flexpart grid
  real               :: dx                       ! longitudinal resolution of flexpart grid
  real               :: dy                       ! latitudinal resolution of flexpart grid
  integer            :: nxshift                  ! longitudinal offset in grid cells relative to 180W for global output
  real, dimension(maxlev) :: outheight           ! height of model layers for grid_init files (m)
  real, dimension(:), allocatable :: glon        ! longitudes of flexpart domain
  real, dimension(:), allocatable :: glat        ! latitudes of flexpart domain

  ! time variables
  ! --------------

  real(kind=8)       :: juldatei                 ! julian start date
  real(kind=8)       :: juldatef                 ! julian end date
  integer            :: ntime                    ! number flux time steps

  ! grid variables
  ! --------------

  real               :: rllx                      ! lower right longitude of inversion domain
  real               :: rlly                      ! lower right latitude of inversion domain
  real               :: rurx                      ! upper right longitude of inversion domain
  real               :: rury                      ! upper right latitude of inversion domain
  real               :: rdx                       ! longitudinal resolution in inversion domain
  real               :: rdy                       ! latitudinal resolution in inversion domain
  integer            :: nxregrid                  ! number of longitudinal grid cells in inversion domain 
  integer            :: nyregrid                  ! number of latitudinal grid cells in inversion domain
  real, dimension(:), allocatable   :: reg_lon    ! longitudes of inversion domain
  real, dimension(:), allocatable   :: reg_lat    ! latitudes of inversion domain
  real, dimension(:,:), allocatable :: lsm        ! global land-sea mask  

  ! region variables
  ! ----------------

  logical            :: useflux                   ! use prior fluxes in grid definition
  logical            :: usezoom                   ! define global grid but with fine resolution only over inversion domain
  logical            :: uselanduse                ! use land use in grid definition
  real               :: cutoff                    ! fractile of sensitivity giving cutoff in grid definition
  integer            :: nmap                      ! number of resolution steps
  integer, dimension(:), allocatable   :: mapdec  ! resolution steps
  integer                              :: nbox    ! number of boxes in inversion domain
  real, dimension(:), allocatable      :: area_box! area of boxes
  real               :: zllx                      ! lower right longitude of zoom domain (only if usezoom)
  real               :: zlly                      ! lower right latitude of zoom domain (only if usezoom)
  real               :: zurx                      ! upper right longitude of zoom domain (only if usezoom)
  real               :: zury                      ! upper right latitude of zoom domain (only if usezoom)

  ! observation variables
  ! ---------------------

  integer                              :: nrec    ! number of receptor sites or campaigns
  integer                              :: nobs    ! total number of observations
  character(recname_len), dimension(:), allocatable :: recname ! names of receptors
 
end module mod_var


